package com.alihealth.d2d.provtest.utils;

import com.alihealth.d2d.provtest.factory.EntityFactory;
import com.alihealth.d2d.provtest.factory.EntityFactoryManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.ByteArrayInputStream;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Jason
 * @date 2025年05月29日 10:27:04
 */
public class XmlStringParser {

    private static final Logger logger = LoggerFactory.getLogger(XmlStringParser.class);


    private static String getElementText(Element parent, String tagName) {
        NodeList nodeList = parent.getElementsByTagName(tagName);
        if (nodeList.getLength() > 0) {
            Node node = nodeList.item(0);
            return node.getTextContent().trim();
        }
        return null;
    }

    private static LocalDate parseDate(String dateStr) {
        if (dateStr == null || dateStr.length() != 8 || "无".equals(dateStr)) {
            return null;
        }

        try {
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyyMMdd");
            return LocalDate.parse(dateStr, formatter);
        } catch (Exception e) {
            logger.warn("日期解析失败: " + dateStr, e);
            return null;
        }
    }


    public static <T> void parseXml(String xmlString, String subType, EntityProcessor<T> processor) {
        List<Object> resultList = new ArrayList<>();

        try {
            // 获取对应子类型的工厂
//            EntityFactory factory = EntityFactoryManager.getFactory(subType);
//
//            // 创建DocumentBuilder
//            DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
//            DocumentBuilder builder = dbFactory.newDocumentBuilder();
//
//            // 将字符串转换为输入流
//            ByteArrayInputStream input = new ByteArrayInputStream(xmlString.getBytes("UTF-8"));
//
//            // 解析XML
//            Document document = builder.parse(input);
//
//            // 获取所有data节点
//            NodeList dataNodes = document.getElementsByTagName("data");
//
//            for (int i = 0; i < dataNodes.getLength(); i++) {
//                Node dataNode = dataNodes.item(i);
//                if (dataNode.getNodeType() == Node.ELEMENT_NODE) {
//                    Element dataElement = (Element) dataNode;
//
//                    // 创建实体对象
//                    Object entity = factory.createEntity();
//
//                    // 填充实体属性
//                    factory.populateEntity(entity, dataElement);
//
//                    resultList.add(entity);
//                }
//            }

            EntityFactory factory = EntityFactoryManager.getFactory(subType);
            DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = dbFactory.newDocumentBuilder();
            ByteArrayInputStream input = new ByteArrayInputStream(xmlString.getBytes("UTF-8"));
            Document document = builder.parse(input);
            NodeList dataNodes = document.getElementsByTagName("data");

            for (int i = 0; i < dataNodes.getLength(); i++) {
                Node dataNode = dataNodes.item(i);
                if (dataNode.getNodeType() == Node.ELEMENT_NODE) {
                    Element dataElement = (Element) dataNode;
                    @SuppressWarnings("unchecked")
                    T entity = (T) factory.createEntity();
                    factory.populateEntity(entity, dataElement);
                    processor.process(entity);
                }
            }
        } catch (Exception e) {
            logger.error("解析XML字符串出错", e);
        }

        //return resultList;
    }
}
